#include <TMB.hpp>
/*
 * this function estimates the constant values for all the teams using the Generalized Poisson 
 */

template<class Type>
Type objective_function<Type>::operator() ()
{
  DATA_VECTOR(x);//the  home goals for all the seasons 
  DATA_VECTOR(y); // the away goals for all the sesons
  DATA_FACTOR(home); // home team, of the possible 38 teams
  DATA_FACTOR(away); // away teams
  DATA_MATRIX(DT);
  DATA_FACTOR(mh);
  DATA_FACTOR(ma);
  DATA_INTEGER(N);
  DATA_INTEGER(nt);
  
  PARAMETER_VECTOR(a0); // attacking strength for the teams
  PARAMETER_VECTOR(d0); // defensive strength for the teams
  PARAMETER_VECTOR(log_sd);
  PARAMETER_MATRIX(a);
  PARAMETER_MATRIX(d);
  PARAMETER(tau);
  PARAMETER(c);
  PARAMETER(ch);
  PARAMETER(delta);
  
  
  Type nll = 0;
  Type lamH;
  Type lamA;
  
  nll -= sum(dnorm(a0,0,exp(log_sd[0]),true));
  nll -= sum(dnorm(d0,0,exp(log_sd[1]),true));
  
  for(int i = 0; i < nt;i++){
    nll -= dnorm(a(i,0),a0[i],DT(i,0)*exp(log_sd[0])/tau,true);
    nll -= dnorm(d(i,0),d0[i],DT(i,0)*exp(log_sd[1])/tau,true);
    for(int j = 1;j < N;j++){
      nll -= dnorm(a(i,j),a(i,j-1),(DT(i,j))*exp(log_sd[0])/tau,true);
      nll -= dnorm(d(i,j),d(i,j-1),(DT(i,j))*exp(log_sd[1])/tau,true);
      
    }
  }
  
  for(int i = 0; i< x.size();i++){
    if(mh[i]==0&&ma[i]==0){
      lamH = exp(c+ch+a0[home[i]]-d0[away[i]]);
      lamA = exp(c+a0[away[i]]-d0[home[i]]);
      nll -= log(lamH)+ (x[i]-1)*log(lamH+delta*x[i])-lamH-delta*x[i]-lfactorial(x[i]);
      nll -= log(lamA)+ (y[i]-1)*log(lamA+delta*y[i])-lamA-delta*y[i]-lfactorial(y[i]);
      
    }
    else if(mh[i]==0&&ma[i]!=0){
      lamH = exp(c+ch+a0[home[i]]-d(away[i],ma[i]-1));
      lamA = exp(c+a(away[i],ma[i]-1)-d0[home[i]]);
      nll -= log(lamH)+ (x[i]-1)*log(lamH+delta*x[i])-lamH-delta*x[i]-lfactorial(x[i]);
      nll -= log(lamA)+ (y[i]-1)*log(lamA+delta*y[i])-lamA-delta*y[i]-lfactorial(y[i]);
    }
    else if(mh[i]!=0&&ma[i]==0){
      lamH = exp(c+ch+a(home[i],mh[i]-1)-d0[away[i]]);
      lamA = exp(c+a0[away[i]]-d(home[i],mh[i]-1));
      nll -= log(lamH)+ (x[i]-1)*log(lamH+delta*x[i])-lamH-delta*x[i]-lfactorial(x[i]);
      nll -= log(lamA)+ (y[i]-1)*log(lamA+delta*y[i])-lamA-delta*y[i]-lfactorial(y[i]);

    }
    else {
      lamH = exp(c+ch+a(home[i],mh[i]-1)-d(away[i],ma[i]-1));
      lamA = exp(c+a(away[i],ma[i]-1)-d(home[i],mh[i]-1));
      nll -= log(lamH)+ (x[i]-1)*log(lamH+delta*x[i])-lamH-delta*x[i]-lfactorial(x[i]);
      nll -= log(lamA)+ (y[i]-1)*log(lamA+delta*y[i])-lamA-delta*y[i]-lfactorial(y[i]);
      
    }
  }
  
  return nll;
}