#include <TMB.hpp>
/*
 * this file uses brownian motion to model the strengths of the team over time. It includes the memory of the brownian motion as a factor
 * it also lets the home field advantage change with the seasons or more importantly change when the suporters dissaper from the stands
 */
template<class Type>
Type objective_function<Type>::operator() ()
{



  DATA_VECTOR(x);//the  home goals 
  DATA_VECTOR(y); // the away goals 
  DATA_FACTOR(home); // home team, of the possible nt teams
  DATA_FACTOR(away); // away teams, of the possible nt teams
  DATA_MATRIX(DT); // each row says how many days thers is between two matches for a the team. 
  DATA_FACTOR(mh);//witch number the match is for the team that is playing at home
  DATA_FACTOR(ma);// witch number the match is for the team that is playig away
  DATA_INTEGER(N); // the highest number of matches played by any team
  DATA_INTEGER(nt);// the number of teams that playes in the data set
  DATA_INTEGER(tau);// the memory of the brownian motion
  DATA_FACTOR(sos);//season number
  
  PARAMETER_VECTOR(a0); // attacking strength for the teams at the begining
  PARAMETER_VECTOR(d0); // defensive strength for the teams at the begining
  PARAMETER_VECTOR(log_sd);// the standard deviation for the random intercept
  PARAMETER_MATRIX(a);//each row is the attacking strenght of the team over time
  PARAMETER_MATRIX(d);//each row is the defending strenght of the team over time
  PARAMETER_VECTOR(c);//the constant 
  PARAMETER_VECTOR(ch);//the home field advantage
  
  
  Type nll = 0; // the negative log likelihood
  
  nll -= sum(dnorm(a0,0,exp(log_sd[0]),true));//the random intercpts at the begingig
  nll -= sum(dnorm(d0,0,exp(log_sd[1]),true));
  
  
  //the browian motion
  for(int i = 0; i < nt;i++){
    nll -= dnorm(a(i,0),a0[i],DT(i,0)*exp(log_sd[0])/tau,true);
    nll -= dnorm(d(i,0),d0[i],DT(i,0)*exp(log_sd[1])/tau,true);
    for(int j = 1;j < N;j++){
      nll -= dnorm(a(i,j),a(i,j-1),(DT(i,j))*exp(log_sd[0])/tau,true);
      nll -= dnorm(d(i,j),d(i,j-1),(DT(i,j))*exp(log_sd[1])/tau,true);
      
    }
  }
  /*
   * this is the poisson process. the reason for all the if else statements is that  when a team plaues its first match a0[] and d0[] is
   * used. and later a[,mh] and d[,mh] is used. 
   */
  
  
  for(int i = 0; i< x.size();i++){
    if(mh[i]==0&&ma[i]==0){
      nll -= dpois(x[i],exp(c[sos[i]]+ch[sos[i]]+a0[home[i]]-d0[away[i]]),true);
      nll -= dpois(y[i],exp(c[sos[i]]+a0[away[i]]-d0[home[i]]),true);
    }
    else if(mh[i]==0&&ma[i]!=0){
      nll -= dpois(x[i],exp(c[sos[i]]+ch[sos[i]]+a0[home[i]]-d(away[i],ma[i]-1)),true);
      nll -= dpois(y[i],exp(c[sos[i]]+a(away[i],ma[i]-1)-d0[home[i]]),true);
    }
    else if(mh[i]!=0&&ma[i]==0){
      nll -= dpois(x[i],exp(c[sos[i]]+ch[sos[i]]+a(home[i],mh[i]-1)-d0[away[i]]),true);
      nll -= dpois(y[i],exp(c[sos[i]]+a0[away[i]]-d(home[i],mh[i]-1)),true);
    }
    else {
      nll -= dpois(x[i],exp(c[sos[i]]+ch[sos[i]]+a(home[i],mh[i]-1)-d(away[i],ma[i]-1)),true);
      nll -= dpois(y[i],exp(c[sos[i]]+a(away[i],ma[i]-1)-d(home[i],mh[i]-1)),true);
    }
  }
  
  return nll;
}