#include <TMB.hpp>
/*
 * This model uses the Generalized Poisson instead of the normal poisson. 
 */

template<class Type>
Type objective_function<Type>::operator() ()
{
  DATA_VECTOR(x); // home goals
  DATA_VECTOR(y); // away goals
  DATA_FACTOR(home); // what teams playes at home
  DATA_FACTOR(away); // what teams playes away in match i
  DATA_INTEGER(N); //the number of teams 
  
  PARAMETER_VECTOR(log_sd); // log of the sd for attack and defence
  PARAMETER_VECTOR(a); // the attack for each of the 20 teams in all 38 rounds
  PARAMETER_VECTOR(d); // same but for defence
  PARAMETER(c);  // the intercept for goals
  PARAMETER(ch);  // the added effect of home field, expected to be higher than 0
  PARAMETER(delta);
  
  
  Type nll = 0;
  vector<Type> residual(2);
  vector<Type> sd = exp(log_sd);
  matrix<Type> Sigma(2, 2); // the varaincematrix for a,d
  Sigma.row(0) << sd[0]*sd[0],     sd[1]*sd[0]*0;
  Sigma.row(1) << sd[0]*sd[1]*0, sd[1]*sd[1];
  
  
  using namespace density;
  MVNORM_t<Type> neg_log_dmvnormI(Sigma);
  //initiating the X_0. 
  for(int i = 0;i < N;i++){
    residual[0] = a[i];
    residual[1] = d[i];
    nll += neg_log_dmvnormI(residual);
  }
  
  
  Type lamH;
  Type lamA;
  // the generalized poisson model
  for(int i = 0; i < x.size();i++){
    lamH = exp(a[home[i]]-d[away[i]]+c+ch);
    lamA = exp(a[away[i]]-d[home[i]]+c);
    nll -= log(lamH)+ (x[i]-1)*log(lamH+delta*x[i])-lamH-delta*x[i]-lfactorial(x[i]);
    nll -= log(lamA)+ (y[i]-1)*log(lamA+delta*y[i])-lamA-delta*y[i]-lfactorial(y[i]);
  }
  
  
  return nll;
}