#include <TMB.hpp>
/*
 * this file uses the poisson distribution to model the number of goals scored by the different teams in the premier league. 
 */

template<class Type>
Type objective_function<Type>::operator() ()
{
  DATA_VECTOR(x); // home goals
  DATA_VECTOR(y); // away goals
  DATA_FACTOR(home); // what teams playes at home
  DATA_FACTOR(away); // what teams playes away in match i
  DATA_INTEGER(N);// the number of teams
  
  PARAMETER_VECTOR(log_sd); // log of the sd for attack and defence
  PARAMETER_VECTOR(a); // the attack for each of the 20 teams
  PARAMETER_VECTOR(d); // same but for defence
  PARAMETER(c);  // the intercept for goals
  PARAMETER(ch);  // the added effect of home field, expected to be higher than 0
  
  
  Type nll = 0;
  vector<Type> residual(2);
  vector<Type> sd = exp(log_sd);
  matrix<Type> Sigma(2, 2); // the varaincematrix for a,d
  Sigma.row(0) << sd[0]*sd[0],     sd[1]*sd[0]*0;
  Sigma.row(1) << sd[0]*sd[1]*0, sd[1]*sd[1];
  
  
  using namespace density;
  MVNORM_t<Type> neg_log_dmvnormI(Sigma);
  //initiating the X_0. 
  for(int i = 0;i < N;i++){
    residual[0] = a[i];
    residual[1] = d[i];
    nll += neg_log_dmvnormI(residual);
  }
  
  
  
  // the poisson model
  for(int i = 0; i < x.size();i++){
    nll -= dpois(x[i],exp(a[home[i]]-d[away[i]]+c+ch),true);
    nll -= dpois(y[i],exp(a[away[i]]-d[home[i]]+c),true);
  }
  
  
  return nll;
}
