#include <TMB.hpp>
/*
 * this model uses the generalized bradley terry model, a brownian motion is used for the change in strength over time.
 */
template<class Type>
Type objective_function<Type>::operator() ()
{
  DATA_MATRIX(X);//the results of the match. Each row is one match. (0,0,1) is home victory.
  //DATA_MATRIX(X);
  DATA_FACTOR(home); // the home team 0-19 in alphabitical order
  DATA_FACTOR(away); // the away teams
  DATA_FACTOR(mh); // match number for the different games, home team
  DATA_FACTOR(ma);
  DATA_MATRIX(DT);//each row has the number of dayes between games for the team. 
  DATA_INTEGER(nt);//the number of teams
  DATA_INTEGER(N);//the number of games for the team that has played the most games.
  
  PARAMETER(theta);
  PARAMETER(h);//home field advantage
  PARAMETER_VECTOR(lam1);//the strength of the teams the first round
  PARAMETER_MATRIX(lam);//each row is one teams strength at differetn matches
  PARAMETER(log_sigma);//the variance across teams
  PARAMETER(tau);//how fast the teams strength change over time
  
  Type nll = 0;
  vector<Type> prob(3);
  vector<Type> res(3);
  
  nll -= sum(dnorm(lam1,0,exp(log_sigma),true));
  
  
  for(int i = 0; i < nt;i++){
    nll -= dnorm(lam(i,0),lam1[i],DT(i,0)*exp(log_sigma)/tau,true);
    for(int j = 1;j < N;j++){
      nll -= dnorm(lam(i,j),lam(i,j-1),(DT(i,j))*exp(log_sigma)/tau,true);
    }
  }

  
  for(int i = 0; i<home.size();i++){
    if(mh[i]==0&&ma[i]==0){
      prob[0] = 1/(1+exp(-(lam1[home[i]]-lam1[away[i]]-theta+h)));
      prob[1] = 1/(1+exp(-(lam1[home[i]]-lam1[away[i]]+theta+h)))-1/(1+exp(-(lam1[home[i]]-lam1[away[i]]-theta+h)));
      prob[2] = 1-1/(1+exp(-(lam1[home[i]]-lam1[away[i]]+theta+h)));
      res = X.row(i);
      nll -= dmultinom(res,prob,true);
    }
    else if(mh[i]==0&&ma[i]!=0){
      prob[0] = 1/(1+exp(-(lam1[home[i]]-lam(away[i],ma[i]-1)-theta+h)));
      prob[1] = 1/(1+exp(-(lam1[home[i]]-lam(away[i],ma[i]-1)+theta+h)))-1/(1+exp(-(lam1[home[i]]-lam(away[i],ma[i]-1)-theta+h)));
      prob[2] = 1-1/(1+exp(-(lam1[home[i]]-lam(away[i],ma[i]-1)+theta+h)));
      res = X.row(i);
      nll -= dmultinom(res,prob,true);
    }
    else if(mh[i]!=0&&ma[i]==0){
      prob[0] = 1/(1+exp(-(lam(home[i],mh[i]-1)-lam1[away[i]]-theta+h)));
      prob[1] = 1/(1+exp(-(lam(home[i],mh[i]-1)-lam1[away[i]]+theta+h)))-1/(1+exp(-(lam(home[i],mh[i]-1)-lam1[away[i]]-theta+h)));
      prob[2] = 1-1/(1+exp(-(lam(home[i],mh[i]-1)-lam1[away[i]]+theta+h)));
      res = X.row(i);
      nll -= dmultinom(res,prob,true);
    }
    else {
      prob[0] = 1/(1+exp(-(lam(home[i],mh[i]-1)-lam(away[i],ma[i]-1)-theta+h)));
      prob[1] = 1/(1+exp(-(lam(home[i],mh[i]-1)-lam(away[i],ma[i]-1)+theta+h)))-1/(1+exp(-(lam(home[i],mh[i]-1)-lam(away[i],ma[i]-1)-theta+h)));
      prob[2] = 1-1/(1+exp(-(lam(home[i],mh[i]-1)-lam(away[i],ma[i]-1)+theta+h)));
      res = X.row(i);
      nll -= dmultinom(res,prob,true);
    }
  }
   

  
  return nll;
}